/*
 * Level.cpp
 * 
 * Copyright (C) 2008 Matthias Schiffer <matthias@gamezock.de>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Level.h"
#include "Room.h"
#include "Gate.h"
#include "Texture.h"

namespace ZoomEdit {
namespace Data {

Level::Level(xmlpp::Element *levelNode) : info(dynamic_cast<xmlpp::Element*>(levelNode->get_children("info").front())) {
  roomsNode = dynamic_cast<xmlpp::Element*>(levelNode->get_children("rooms").front());
  gatesNode = dynamic_cast<xmlpp::Element*>(levelNode->get_children("gates").front());
  texturesNode = dynamic_cast<xmlpp::Element*>(levelNode->get_children("textures").front());
  
  xmlpp::Node::NodeList nodeList = roomsNode->get_children("room");
  
  for(xmlpp::Node::NodeList::iterator room = nodeList.begin(); room != nodeList.end(); ++room) {
    xmlpp::Element *roomNode = dynamic_cast<xmlpp::Element*>(*room);
    
    if(roomNode)
      rooms.push_back(new Room(roomNode));
  }
  
  nodeList = gatesNode->get_children("gate");
  
  for(xmlpp::Node::NodeList::iterator gate = nodeList.begin(); gate != nodeList.end(); ++gate) {
    xmlpp::Element *gateNode = dynamic_cast<xmlpp::Element*>(*gate);
    
    if(gateNode)
      gates.push_back(new Gate(gateNode));
  }
  
  nodeList = texturesNode->get_children("texture");
  
  for(xmlpp::Node::NodeList::iterator tex = nodeList.begin(); tex != nodeList.end(); ++tex) {
    xmlpp::Element *texNode = dynamic_cast<xmlpp::Element*>(*tex);
    
    if(texNode)
      textures.push_back(new Texture(texNode));
  }
}

Level::~Level() {
  for(std::list<Room*>::iterator room = rooms.begin(); room != rooms.end(); ++room)
    delete *room;
}

Room* Level::getRoom(const Glib::ustring &id) const {
  for(std::list<Room*>::const_iterator room = rooms.begin(); room != rooms.end(); ++room) {
    if((*room)->getId() == id)
      return *room;
  }
  
  return 0;
}

Room* Level::addRoom(const Glib::ustring &id) {
  xmlpp::Element *node = roomsNode->add_child("room");
  
  node->set_attribute("id", id);
  
  Room *room = new Room(node);
  rooms.push_back(room);
  
  return room;
}

void Level::removeRoom(Room *room) {
  roomsNode->remove_child(room->getNode());
  rooms.remove(room);
  delete room;
}

Gate* Level::getGate(const Glib::ustring &id) const {
  for(std::list<Gate*>::const_iterator gate = gates.begin(); gate != gates.end(); ++gate) {
    if((*gate)->getId() == id)
      return *gate;
  }
  
  return 0;
}

Gate* Level::addGate(const Glib::ustring &id) {
  xmlpp::Element *node = gatesNode->add_child("gate");
  
  node->set_attribute("id", id);
  node->set_attribute("room1", "");
  node->set_attribute("room2", "");
  
  Gate *gate = new Gate(node);
  gates.push_back(gate);
  
  return gate;
}

void Level::removeGate(Gate *gate) {
  gatesNode->remove_child(gate->getNode());
  gates.remove(gate);
  delete gate;
}

Texture* Level::getTexture(const Glib::ustring &id) const {
  for(std::list<Texture*>::const_iterator tex = textures.begin(); tex != textures.end(); ++tex) {
    if((*tex)->getId() == id)
      return *tex;
  }
  
  return 0;
}

Texture* Level::addTexture(const Glib::ustring &id) {
  xmlpp::Element *node = texturesNode->add_child("texture");
  
  node->set_attribute("id", id);
  node->set_attribute("name", "");
  
  Texture *tex = new Texture(node);
  textures.push_back(tex);
  
  return tex;
}

void Level::removeTexture(Texture *tex) {
  texturesNode->remove_child(tex->getNode());
  textures.remove(tex);
  delete tex;
}

}
}
