/*
 * Instance.cpp
 * 
 * Copyright (C) 2008 Matthias Schiffer <matthias@gamezock.de>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Instance.h"
#include <iostream>
#include <libxml++/validators/dtdvalidator.h>
#include <libxml/tree.h>
#include <gtkmm/toolbutton.h>
#include <Gui/Window.h>
#include <Gui/RenderArea.h>
#include <Data/Level.h>
#include <View/TopView.h>
#include <View/MapView.h>

namespace ZoomEdit {

guint Instance::instances = 0;

Instance::Instance(const Glib::ustring &file) : window(0), levelXml(0), level(0), view(0) {
  instances++;
  
#ifdef GLIBMM_EXCEPTIONS_ENABLED
  try
  {
    xml = Gnome::Glade::Xml::create("zoomedit.glade");
  }
  catch(const Gnome::Glade::XmlError& ex)
  {
    std::cerr << ex.what() << std::endl;
    return;
  }
#else
  std::auto_ptr<Gnome::Glade::XmlError> error;
  xml = Gnome::Glade::Xml::create("zoomedit.glade", "", "", error);
  if(error.get())
  {
    std::cerr << error->what() << std::endl;
    return;
  }
#endif
  
  xml->get_widget_derived("WindowMain", window);
  if(!window) {
    xml.clear();
    return;
  }
  
  window->signal_hide().connect(sigc::mem_fun(this, &Instance::destroy));
  
  view = new View::TopView();
  window->getRenderArea()->setView(view);
  
  mapView = new View::MapView(view);
  window->getMapArea()->setView(mapView);
  
  Gtk::ToolButton *button;
  xml->get_widget("ToolButtonZoomIn", button);
  button->signal_clicked().connect(sigc::bind(sigc::mem_fun(view, &View::TopView::zoom), 2, 0, 0));
  
  xml->get_widget("ToolButtonZoomOut", button);
  button->signal_clicked().connect(sigc::bind(sigc::mem_fun(view, &View::TopView::zoom), -2, 0, 0));
  
  if(file.empty())
    createLevel();
  else
    loadLevel(file);
  
  window->show();
}

Instance::~Instance() {
  if(window)
    delete window;
  
  if(mapView)
    delete mapView;
  
  if(view)
    delete view;
  
  if(level)
    delete level;
  
  if(levelXml)
    delete levelXml;
  
  instances--;
  if(!instances)
    Gtk::Main::quit();
}

void Instance::createLevel() {
  if(level) {
    view->setLevel(0);
    delete level;
  }
  
  if(levelXml)
    delete levelXml;
  
  levelXml = new xmlpp::DomParser;
  xmlpp::Document *doc = levelXml->get_document();
  
  xmlSetDocCompressMode(doc->cobj(), 9);
  
  xmlpp::Element *root = doc->create_root_node("level");
  
  xmlpp::Element *info = root->add_child("info");
  info->add_child("name");
  info->add_child("desc");
  xmlpp::Element *start = info->add_child("start");
  start->set_attribute("x", "0");
  start->set_attribute("y", "0");
  start->set_attribute("z", "0");
  
  root->add_child("rooms");
  root->add_child("gates");
  root->add_child("textures");
  
  level = new Data::Level(root);
  
  view->setLevel(level);
}

bool Instance::loadLevel(const Glib::ustring &file) {
  if(level) {
    view->setLevel(0);
    delete level;
    level = 0;
  }
  
  if(levelXml)
    delete levelXml;
  
  levelXml = new xmlpp::DomParser(file);
  xmlpp::Document *doc = levelXml->get_document();
  
  if(!doc || !doc->get_root_node())
    return false;
  
  if(!xmlpp::DtdValidator("level.dtd").validate(doc))
    return false;
  
  level = new Data::Level(doc->get_root_node());
  
  view->setLevel(level);
  
  return true;
}

bool Instance::saveLevel(const Glib::ustring &file) {
  if(!level | !levelXml->get_document())
    return false;
  
  levelXml->get_document()->write_to_file(file);
  return true;
}

bool Instance::create(const Glib::ustring &file) {
  Instance *instance = new Instance(file);
  
  if(!instance->level) {
    delete instance;
    return false;
  }
  
  return true;
}

void Instance::destroy() {
  delete this;
}

}
